@@ -1,4 +1,5 @@
 use crate::{
+    all_imports::OpSize,
     circuit::EinsumAxes,
     hashmaps::{AHashSet as HashSet, FxHashMap as HashMap},
 };
@@ -128,12 +129,7 @@ pub fn remove_add_few_input(add: &Add) -> Option<CircuitRc> {
     if add.nodes.is_empty() {
         Some(ScalarConstant::new(0f64, add.info().shape.clone(), None).rc())
     } else if add.nodes.len() == 1 {
-        Some(
-            add_make_broadcasts_explicit(add)
-                .unwrap_or(add.clone())
-                .nodes[0]
-                .clone(),
-        )
+        Some(add.nodes[0].clone())
     } else {
         None
     }
@@ -1677,3 +1673,26 @@ pub fn add_fuse_scalar_multiples(add: &Add) -> Option<Add> {
 
     Some(Add::try_new(nodes, add.name_cloned()).unwrap())
 }
+
+#[pyfunction]
+pub fn einsum_permute_to_rearrange(einsum: &Einsum) -> Option<Rearrange> {
+    if einsum.args.len() == 1
+        && einsum.args[0].1.len() == einsum.out_axes.len()
+        && is_unique(&einsum.out_axes)
+        && is_unique(&einsum.args[0].1)
+    {
+        return Some(
+            Rearrange::try_new(
+                einsum.args[0].0.clone(),
+                RearrangeSpec {
+                    input_ints: einsum.args[0].1.iter().map(|z| sv![*z]).collect(),
+                    output_ints: einsum.out_axes.iter().map(|z| sv![*z]).collect(),
+                    int_sizes: sv![OpSize::NONE;einsum.info().rank()],
+                },
+                einsum.name_cloned(),
+            )
+            .unwrap(),
+        );
+    }
+    None
+}
