@@ -1,44 +1,35 @@
 use super::{
-    deep_map, deep_map_op_context_preorder_stoppable, deep_map_preorder, deep_map_unwrap_preorder,
-    prelude::*, visit_circuit, visit_circuit_fallable, HashBytes,
+    deep_map_op_context_preorder_stoppable, deep_map_preorder, deep_map_preorder_unwrap,
+    prelude::*, visit_circuit, HashBytes,
 };
 use crate::hashmaps::{AHashSet as HashSet, FxHashMap as HashMap};
-use crate::pycall;
+use crate::{pycall, pycallable};
+use macro_rules_attribute::apply;
 use pyo3::{pyfunction, PyObject, Python};
 
-pub fn filter_nodes(circuit: CircuitRc, filter: &dyn Fn(CircuitRc) -> bool) -> HashSet<CircuitRc> {
-    let mut result: HashSet<CircuitRc> = HashSet::new();
-    visit_circuit(circuit, |circuit| {
-        if filter(circuit.clone()) {
-            result.insert(circuit);
-        }
-    });
-    result
-}
-
-#[pyfunction]
+#[apply(pycallable)]
 #[pyo3(name = "filter_nodes")]
-pub fn filter_nodes_py(
+pub fn filter_nodes<F>(
     circuit: CircuitRc,
-    filter: PyObject,
-) -> Result<HashSet<CircuitRc>, CircuitConstructionError> {
+    filter: F,
+) -> Result<HashSet<CircuitRc>, CircuitConstructionError>
+where
+    F: Fn((circuit, CircuitRc)) -> Result<bool, _>,
+{
     let mut result: HashSet<CircuitRc> = HashSet::new();
-    let err = visit_circuit_fallable(circuit, |circuit| {
-        let filter_result: Result<_, CircuitConstructionError> =
-            pycall!(filter, (circuit.clone(),), CircuitConstructionError);
-        let filter_result = filter_result?;
-        if filter_result {
+    visit_circuit(circuit, |circuit: CircuitRc| {
+        if filter(circuit.clone())? {
             result.insert(circuit);
         }
         Ok(())
-    });
-    err.map(|_x| result)
+    })?;
+    Ok(result)
 }
 
 #[pyfunction]
 #[pyo3(name = "replace_nodes")]
 pub fn replace_nodes_py(circuit: CircuitRc, map: HashMap<CircuitRc, CircuitRc>) -> CircuitRc {
-    deep_map_unwrap_preorder(circuit, |x: CircuitRc| -> CircuitRc {
+    deep_map_preorder_unwrap(circuit, |x: CircuitRc| -> CircuitRc {
         map.get(&x).cloned().unwrap_or(x)
     })
 }
@@ -60,31 +51,6 @@ pub fn replace_nodes(circuit: CircuitRc, map: &HashMap<HashBytes, CircuitRc>) ->
     .unwrap_or(circuit)
 }
 
-#[pyfunction]
-#[pyo3(name = "deep_map_preorder")]
-pub fn deep_map_preorder_py(
-    circuit: CircuitRc,
-    f: PyObject,
-) -> Result<CircuitRc, CircuitConstructionError> {
-    deep_map_preorder(
-        circuit,
-        |x: CircuitRc| -> Result<CircuitRc, CircuitConstructionError> {
-            pycall!(f, (x,), CircuitConstructionError)
-        },
-    )
-}
-
-#[pyfunction]
-#[pyo3(name = "deep_map")]
-pub fn deep_map_py(circuit: CircuitRc, f: PyObject) -> Result<CircuitRc, CircuitConstructionError> {
-    deep_map(circuit, &|x: CircuitRc| -> Result<
-        CircuitRc,
-        CircuitConstructionError,
-    > {
-        pycall!(f, (x,), CircuitConstructionError)
-    })
-}
-
 #[pyfunction]
 #[pyo3(name = "update_nodes")]
 pub fn update_nodes_py(
