@@ -1,5 +1,15 @@
+use super::{
+    named_axes::set_named_axes, prelude::*, CachedCircuitInfo, PyCircuitBase, Shape,
+    TensorEvalError,
+};
 use crate::hashmaps::FxHashMap as HashMap;
+use crate::lru_cache::TensorCacheRrfs;
 use crate::rrfs::{arrayconstant_from_hash, arrayconstant_from_hash_prefix, save_tensor_rrfs};
+use crate::{
+    circuit_node_auto_impl, circuit_node_extra_impl,
+    py_types::{scalar_to_tensor, PyUuid, Tensor, PY_UTILS},
+    tensor_util::{TorchDeviceDtype, TorchDeviceDtypeOp},
+};
 use crate::{new_rc, pyo3_prelude::*, sv};
 use base16::encode_lower;
 use macro_rules_attribute::apply;
@@ -7,16 +17,6 @@ use num_bigint::BigUint;
 use pyo3::types::{IntoPyDict, PyTuple};
 use uuid::Uuid;
 
-use super::{
-    named_axes::set_named_axes, prelude::*, CachedCircuitInfo, PyCircuitBase, Shape,
-    TensorEvalError,
-};
-use crate::{
-    circuit_node_auto_impl, circuit_node_extra_impl,
-    py_types::{scalar_to_tensor, PyUuid, Tensor, PY_UTILS},
-    tensor_util::{TorchDeviceDtype, TorchDeviceDtypeOp},
-};
-
 use crate::circuit::NamedAxes;
 
 macro_rules! circuit_node_auto_leaf_impl {
@@ -104,6 +104,13 @@ impl ArrayConstant {
         .init_info()
         .unwrap()
     }
+    pub fn from_hash_prefix(
+        name: Option<String>,
+        hash_base16: &str,
+        tensor_cache: &mut Option<TensorCacheRrfs>,
+    ) -> Result<Self, PyErr> {
+        arrayconstant_from_hash_prefix(name, hash_base16, tensor_cache)
+    }
 }
 
 #[pymethods]
@@ -190,8 +197,14 @@ impl ArrayConstant {
     }
 
     #[staticmethod]
-    pub fn from_hash_prefix(name: Option<String>, hash_base16: &str) -> Result<Self, PyErr> {
-        arrayconstant_from_hash_prefix(name, hash_base16)
+    #[pyo3(name = "from_hash_prefix")]
+    pub fn from_hash_prefix_py(
+        name: Option<String>,
+        hash_base16: &str,
+        tensor_cache: Option<TensorCacheRrfs>,
+    ) -> Result<Self, PyErr> {
+        let mut tensor_cache = tensor_cache;
+        arrayconstant_from_hash_prefix(name, hash_base16, &mut tensor_cache)
     }
 }
 
