@@ -11,6 +11,7 @@ use crate::util::{apply_fn_until_none, apply_fn_until_same, mapping_until_end, A
 use crate::{pyo3_prelude::*, timed, timed_value};
 use num_bigint::BigUint;
 
+use super::nb_rewrites::{add_elim_removable_axes_weak, einsum_elim_removable_axes_weak};
 use super::visit_circuit_unwrap;
 use super::{
     algebraic_rewrite::*,
@@ -38,48 +39,48 @@ pub fn compiler_simp_step_py(circ: CircuitRc) -> Option<CircuitRc> {
     compiler_simp_step(&circ, &mut Default::default())
 }
 
+fn simp<'a, T: CircuitNode + Into<Circuit> + Clone>(
+    x: &'a T,
+    fns: &[(&'static str, &dyn Fn(&'a T) -> Option<CircuitRc>)],
+    context: &mut OptimizationContext,
+) -> Option<CircuitRc> {
+    for (name, f) in fns {
+        if let Some(result) = f(x) {
+            if **result == x.clone().c() {
+                println!("{}", stringify!(f));
+                x.clone().rc().compiler_print();
+                result.compiler_print();
+                panic!()
+            }
+            if context.settings.log_simplifications {
+                if context.settings.verbose >= 3 {
+                    println!("{}", name);
+                }
+                context.cache.simplification_log.push(name);
+            }
+            return Some(result);
+        }
+    }
+
+    None
+}
+
+macro_rules! f_wrap {
+    ($f:expr) => {
+        (stringify!($f), &|x| $f(x).map(|v| v.rc()))
+    };
+}
+
+macro_rules! l_wrap {
+    ($f:expr) => {
+        (stringify!($f), &|x| $f(x))
+    };
+}
+
 pub fn compiler_simp_step(
     circuit: &Circuit,
     context: &mut OptimizationContext,
 ) -> Option<CircuitRc> {
-    macro_rules! f_wrap {
-        ($f:expr) => {
-            (stringify!($f), &|x| $f(x).map(|v| v.rc()))
-        };
-    }
-
-    macro_rules! l_wrap {
-        ($f:expr) => {
-            (stringify!($f), &|x| $f(x))
-        };
-    }
-
-    fn simp<'a, T: CircuitNode + Into<Circuit> + Clone>(
-        x: &'a T,
-        fns: &[(&'static str, &dyn Fn(&'a T) -> Option<CircuitRc>)],
-        context: &mut OptimizationContext,
-    ) -> Option<CircuitRc> {
-        for (name, f) in fns {
-            if let Some(result) = f(x) {
-                if **result == x.clone().c() {
-                    println!("{}", stringify!(f));
-                    x.clone().rc().compiler_print();
-                    result.compiler_print();
-                    panic!()
-                }
-                if context.settings.log_simplifications {
-                    if context.settings.verbose >= 3 {
-                        println!("{}", name);
-                    }
-                    context.cache.simplification_log.push(name);
-                }
-                return Some(result);
-            }
-        }
-
-        None
-    }
-
     match &*circuit {
         Circuit::Add(node) => {
             let fns: &[(&'static str, &dyn Fn(_) -> _)] = &[
@@ -103,6 +104,7 @@ pub fn compiler_simp_step(
                 f_wrap!(&einsum_of_permute_merge),
                 f_wrap!(&einsum_merge_scalars),
                 l_wrap!(einsum_pull_removable_axes),
+                f_wrap!(&einsum_permute_to_rearrange),
                 l_wrap!(einsum_pull_scatter),
                 f_wrap!(&einsum_push_down_trace),
                 f_wrap!(&einsum_concat_to_add),
@@ -165,12 +167,6 @@ pub fn compiler_simp_step(
     }
 }
 
-/// Deep simplification strategy
-///
-/// The strategy to apply `compiler_simp_step` to each node in the circuit from the bottom up (post-order).
-/// Every time a node is simplified, we iterate over the children to make sure that any children we haven't
-/// seen before get recursively fully simplified before continuing.
-/// The final result is a fixed point where no further `compiler_simp_step` simplifications are possible.
 #[pyfunction]
 #[pyo3(name = "compiler_simp")]
 pub fn compiler_simp_py(circ: CircuitRc) -> CircuitRc {
@@ -179,11 +175,115 @@ pub fn compiler_simp_py(circ: CircuitRc) -> CircuitRc {
 }
 
 pub fn compiler_simp(circ: CircuitRc, opt_context: &mut OptimizationContext) -> CircuitRc {
+    deep_simp(circ, opt_context, compiler_simp_step)
+}
+
+// original python simp
+// def basic_simp_elem(
+//     c: Circuit,
+//     cum_trivial_expand: bool = False,
+//     cumulant_namer: GetPrefixShortNames = lambda _: None,
+//     cum_expand_through_factored_add_concat: bool = False,
+//     cum_expand_through_diags: bool = True,
+//     use_remove_noop_rearrange: bool = False,  # off by default because this makes cumulant stuff non-deterministic!
+//     use_factoring: bool = False,
+//     use_flatten_adds: bool = True,
+//     use_push_down_permute_via_einsum: bool = False,
+//     excludes: Set[Rewrite] = set(),
+//     extras: List[Rewrite] = [],
+//     until: Callable[[Circuit], bool] = lambda _: False,
+//     print_updates: bool = False,
+// ) -> Circuit:
+//     base_transforms: List[Rewrite] = [
+//         try_eliminate_zeros,
+//         try_remove_add_times_one,
+//         try_remove_single_concat,
+//         try_remove_trivial_index,
+//         try_remove_empty_einsum,
+//         try_drop_zero_add,
+//         try_zero_empty_add,
+//         try_drop_mul_ones,
+//         try_fuse_einsum_rearrange,
+//         try_fuse_permute_einsum,
+//         try_fuse_rearrange,
+//         try_fuse_single_einsum,
+//         try_fuse_einsum_single,
+//         try_rearrange_of_const_to_const,
+//         try_nested_einsum_permute_dups_to_eq,
+//     ]
+
+/// this doesn't have enough options + isn't similar enough to python one. we'll use a HOF for options
+pub fn basic_simp_step(circuit: &Circuit, context: &mut OptimizationContext) -> Option<CircuitRc> {
+    match &*circuit {
+        Circuit::Add(node) => {
+            let fns: &[(&'static str, &dyn Fn(_) -> _)] = &[
+                l_wrap!(&remove_add_few_input),
+                f_wrap!(add_elim_zeros),
+                f_wrap!(add_deduplicate),
+                f_wrap!(add_elim_removable_axes_weak),
+            ];
+            simp(node, fns, context)
+        }
+        Circuit::Einsum(node) => {
+            let fns: &[(&'static str, &dyn Fn(_) -> _)] = &[
+                f_wrap!(&einsum_elim_zero),
+                l_wrap!(einsum_elim_identity),
+                f_wrap!(&einsum_of_permute_merge),
+                f_wrap!(&einsum_elim_removable_axes_weak),
+                f_wrap!(&einsum_permute_to_rearrange),
+            ];
+            simp(node, fns, context)
+        }
+        Circuit::Index(node) => {
+            let fns: &[(&'static str, &dyn Fn(_) -> _)] = &[
+                l_wrap!(index_elim_identity),
+                f_wrap!(&index_fuse),
+                l_wrap!(index_merge_scalar),
+            ];
+            simp(node, fns, context)
+        }
+        Circuit::Rearrange(node) => {
+            let fns: &[(&'static str, &dyn Fn(_) -> _)] = &[
+                l_wrap!(rearrange_elim_identity),
+                f_wrap!(&rearrange_fuse),
+                l_wrap!(rearrange_merge_scalar),
+                f_wrap!(&permute_of_einsum_merge),
+            ];
+            simp(node, fns, context)
+        }
+        Circuit::Concat(node) => {
+            let fns: &[(&'static str, &dyn Fn(&Concat) -> _)] = &[
+                l_wrap!(concat_elim_identity),
+                l_wrap!(concat_merge_uniform),
+                f_wrap!(&concat_drop_size_zero),
+                f_wrap!(&concat_fuse),
+                f_wrap!(&concat_repeat_to_rearrange),
+            ];
+            simp(node, fns, context)
+        }
+        _ => None,
+    }
+}
+
+/// Deep simplification strategy
+///
+/// The strategy to apply `compiler_simp_step` to each node in the circuit from the bottom up (post-order).
+/// Every time a node is simplified, we iterate over the children to make sure that any children we haven't
+/// seen before get recursively fully simplified before continuing.
+/// The final result is a fixed point where no further `compiler_simp_step` simplifications are possible.
+pub fn deep_simp<F>(circ: CircuitRc, opt_context: &mut OptimizationContext, f: F) -> CircuitRc
+where
+    F: Fn(&Circuit, &mut OptimizationContext) -> Option<CircuitRc>,
+{
     /// check if any new children have not been simplified yet, and simplify them if so
-    fn simplify_changed_descendants(
+    fn simplify_changed_descendants<F>(
         circ: CircuitRc,
         context: &mut OptimizationContext,
-    ) -> Option<CircuitRc> {
+        f: &F,
+    ) -> Option<CircuitRc>
+    where
+        F: Fn(&Circuit, &mut OptimizationContext) -> Option<CircuitRc>,
+    {
         // if let Some(changed) = compiler_simp_step(circ){
         //     return simplify_changed_descendants(circ, simplified)
         // }
@@ -191,29 +291,32 @@ pub fn compiler_simp(circ: CircuitRc, opt_context: &mut OptimizationContext) ->
             if context.cache.simplified.contains_key(&x.info().hash) {
                 None
             } else {
-                Some(fully_simplify(x, context))
+                Some(fully_simplify(x, context, f))
             }
         })
         .map(|c| c.rc())
     }
     /// fully simplify a circuit and all its descendants recursively until we hit a fixed point
-    fn fully_simplify(circ: CircuitRc, context: &mut OptimizationContext) -> CircuitRc {
+    fn fully_simplify<F>(circ: CircuitRc, context: &mut OptimizationContext, f: &F) -> CircuitRc
+    where
+        F: Fn(&Circuit, &mut OptimizationContext) -> Option<CircuitRc>,
+    {
         if let Some(result) = context.cache.simplified.get(&circ.info().hash) {
             return result.clone();
         }
         let mut result: CircuitRc = circ
-            .map_children_unwrap(&mut |x: CircuitRc| fully_simplify(x, context))
+            .map_children_unwrap(&mut |x: CircuitRc| fully_simplify(x, context, f))
             .rc();
         for iter_count in 0.. {
-            match compiler_simp_step(&result, context) {
-                Some(r) => result = simplify_changed_descendants(r.clone(), context).unwrap_or(r),
+            match f(&result, context) {
+                Some(r) => {
+                    result = simplify_changed_descendants(r.clone(), context, f).unwrap_or(r)
+                }
                 None => break,
             }
             if iter_count > 50 {
                 result.compiler_print();
-                compiler_simp_step(&result, context)
-                    .unwrap()
-                    .compiler_print();
+                f(&result, context).unwrap().compiler_print();
                 panic!();
             }
         }
@@ -223,7 +326,12 @@ pub fn compiler_simp(circ: CircuitRc, opt_context: &mut OptimizationContext) ->
             .insert(circ.info().hash, result.clone());
         result
     }
-    fully_simplify(circ, opt_context)
+    fully_simplify(circ, opt_context, &f)
+}
+
+#[pyfunction]
+pub fn basic_simp(circuit: CircuitRc) -> CircuitRc {
+    deep_simp(circuit, &mut Default::default(), basic_simp_step)
 }
 
 #[pyfunction]
