@@ -4,14 +4,6 @@ use crate::{
 };
 use std::fmt::{self, Display};
 
-use crate::{
-    cached_lambda,
-    py_types::{ExtraPySelfOps, Tensor},
-    pycall,
-    tensor_util::{Shape, TorchDeviceDtype},
-    timed,
-};
-
 use super::{
     circuit_optimizer::{OptimizationContext, OptimizationSettings},
     circuit_utils::{hash_to_node, toposort_circuit},
@@ -23,12 +15,26 @@ use super::{
     visit_circuit_unwrap, Add, ArrayConstant, HashBytes, IrreducibleNode, Leaf, ModuleNodeSpec,
     ScalarConstant, Symbol,
 };
+use crate::circuit::schedule_send::ScheduleToSend;
+use crate::lru_cache::TensorCacheRrfs;
 use crate::pyo3_prelude::*;
 use crate::util::filter_to_idx;
+use crate::{
+    cached_lambda,
+    py_types::{ExtraPySelfOps, Tensor},
+    pycall,
+    tensor_util::{Shape, TorchDeviceDtype},
+    timed,
+};
 use itertools::Itertools;
 use macro_rules_attribute::apply;
+use miniserde::json;
 use num_bigint::BigUint;
-use pyo3::{create_exception, exceptions::PyException};
+use pyo3::{
+    create_exception,
+    exceptions::PyException,
+    types::{PyByteArray, PyBytes},
+};
 use thiserror::Error;
 
 #[derive(Clone, Debug)]
@@ -184,6 +190,46 @@ impl Schedule {
             .unwrap_or(&0)
             + 1
     }
+
+    pub fn serialize(&self) -> String {
+        let tosend: ScheduleToSend = self.into();
+        json::to_string(&tosend)
+    }
+
+    #[staticmethod]
+    #[pyo3(name = "deserialize")]
+    pub fn deserialize_py(
+        string: String,
+        device: String,
+        tensor_cache: Option<TensorCacheRrfs>,
+    ) -> Result<Self, CircuitConstructionError> {
+        let mut tensor_cache = tensor_cache;
+        Schedule::deserialize(string, device, &mut tensor_cache)
+    }
+
+    pub fn tosend(&self) -> ScheduleToSend {
+        self.into()
+    }
+
+    pub fn evaluate_remote(&self, remote_url: String) -> Option<Tensor> {
+        let tosend: ScheduleToSend = self.into();
+        tosend.evaluate_remote(remote_url, self.device_dtype.device.clone())
+    }
+    pub fn evaluate_remote_many(&self, remote_url: String) -> Option<Vec<Tensor>> {
+        let tosend: ScheduleToSend = self.into();
+        tosend.evaluate_remote_many(remote_url, self.device_dtype.device.clone())
+    }
+}
+impl Schedule {
+    pub fn deserialize(
+        string: String,
+        device: String,
+        tensor_cache: &mut Option<TensorCacheRrfs>,
+    ) -> Result<Self, CircuitConstructionError> {
+        let sent: ScheduleToSend = json::from_str(&string)
+            .map_err(|e| CircuitConstructionError::ParsingFail { string: string })?;
+        sent.load(device, tensor_cache)
+    }
 }
 
 impl Display for Schedule {
@@ -321,6 +367,10 @@ fn identity(key: usize, shape: Shape) -> CircuitRc {
     )
 }
 
+pub fn get_children_keys(circuit: CircuitRc) -> Vec<usize> {
+    circuit.children().map(get_child_key).collect()
+}
+
 pub fn evaluate_schedule(schedule: &Schedule) -> HashMap<usize, Tensor> {
     let mut live: HashMap<usize, Tensor> = schedule
         .constants
